import pandas as pd
from datetime import datetime as dtdt
from os.path import expanduser
import os
import yaml
import subprocess


def reset_jikanwari():
    jigen = {
        1: "08:30~10:25",
        2: "10:25~12:10",
        3: "13:00~14:55",
        4: "14:55~16:50",
        5: "16:50~18:45",
        6: "18:45~20:30",
    }
    jigen = pd.DataFrame(jigen, index=["jigen"]).T
    dirname = []
    for i in range(22, 29):
        dirname.append(dtdt(*[2020, 6, i]).strftime("%a"))
    jikanwari = [[d + str(i) for d in dirname]for i in range(1, 7)]
    jikanwari = pd.DataFrame(
        jikanwari,
        columns=dirname,
        index=[i for i in range(1, 7)])
    home = expanduser("~")
    base = os.path.join(home, ".myscreenshot")
    os.makedirs(base, exist_ok=True)
    jikanwari_path = os.path.join(base, "jikanwari.csv")
    pd.concat([jigen, jikanwari], axis=1).to_csv(jikanwari_path)


def reset_settings():
    home = expanduser("~")
    base = os.path.join(home, ".myscreenshot")
    setting_path = os.path.join(base, "settings.yml")
    res = subprocess.run(
        "defaults read com.apple.screencapture location",
        shell=True,
        capture_output=True)
    if res.returncode == 0:
        dirname = res.stdout[:-1].decode()
    else:
        dirname = "~/Desktop"
    res = subprocess.run(
        "defaults read com.apple.screencapture name",
        shell=True,
        capture_output=True)
    if res.returncode == 0:
        prefix = res.stdout[:-1].decode()
    else:
        prefix = "スクリーンショット"
    with open(setting_path, "w") as yf:
        yaml.dump({
            "MOVE": {
                "prefix": prefix,
                "dirname": dirname,
                "savedirname": "~/Desktop"

            }
        }, yf, default_flow_style=False)


def change_jikanwari():
    home = expanduser("~")
    base = os.path.join(home, ".myscreenshot")
    jikanwari_path = os.path.join(base, "jikanwari.csv")
    res = subprocess.run(f"open {jikanwari_path} -a 'Microsoft Excel'",
                         shell=True)
    if res.returncode != 0:
        res = subprocess.run(f"open {jikanwari_path}", shell=True)


def change_savedir():
    home = expanduser("~")
    base = os.path.join(home, ".myscreenshot")
    with open(os.path.join(base, "settings.yml"), "r") as f:
        cfgold = yaml.load(f, Loader=yaml.FullLoader)
    reset_settings()
    setting_path = os.path.join(base, "settings.yml")
    with open(os.path.join(base, "settings.yml"), "r") as f:
        cfg = yaml.load(f, Loader=yaml.FullLoader)
    savedirname = input(
        f"[now:{cfg['MOVE']['savedirname']}]:press enter or input savedir path")
    if savedirname == "":
        cfg["MOVE"]["savedirname"] = cfgold["MOVE"]["savedirname"]
    elif os.path.isdir(savedirname):
        cfg["MOVE"]["savedirname"] = savedirname
    else:
        flg = "No"
        while True:
            flg = input(f"create {savedirname}:Yes/[No]")
            if flg == "Yes" or flg == "No":
                break
            else:
                print("invalid input:type Yes or No")
        if flg == "Yes":
            os.makedirs(savedirname)
            cfg["MOVE"]["savedirname"] = savedirname
        else:
            cfg["MOVE"]["savedirname"] = cfgold["MOVE"]["savedirname"]
    with open(setting_path, "w") as yf:
        yaml.dump(
            cfg,
            yf,
            default_flow_style=False
        )


def init():
    reset_jikanwari()
    reset_settings()
    change_savedir()
    change_jikanwari()



from absl import app
from absl import flags
import time
from os.path import expanduser
import os
import signal
from .mv_screenshot import mv_default
from .settings import init, change_savedir, change_jikanwari
from .daemonize import interval_run, start_daemon, log_to_stdout


FLAGS = flags.FLAGS
flags.DEFINE_bool(
    'reset',
    False,
    'reset all settings of this program',
    short_name='r')
flags.DEFINE_bool(
    'settings',
    False,
    'edit save directory',
    short_name='es')
flags.DEFINE_bool(
    'jikanwari',
    False,
    'edit jikanwari',
    short_name='ej')
flags.DEFINE_bool(
    'startdaemon',
    False,
    'daemonize program',
    short_name='start')
flags.DEFINE_bool(
    'stopdaemon',
    False,
    'stop daemon',
    short_name='kill')
flags.DEFINE_bool(
    'test',
    True,
    'test',
    short_name='t')
flags.DEFINE_bool(
    'exec',
    True,
    'test',
    short_name='e')


def screenshot_daemon(arg=None, kws=None, pf=log_to_stdout):
    while True:
        mv_default()
        time.sleep(60)


home = expanduser("~")
base = os.path.join(home, ".myscreenshot")
pidpath = os.path.join(base, "python_daemon.pid")
start_daemon = start_daemon(log_to_stdout)
screenshot_daemon = interval_run(1)(screenshot_daemon)


def main(argv):
    if FLAGS.jikanwari:
        change_jikanwari()
        return 0
    if FLAGS.reset:
        init()
    if FLAGS.settings:
        change_savedir()
    if FLAGS.stopdaemon:
        if not os.path.isfile(pidpath):
            return None
        with open(pidpath, "rb") as f:
            pid = f.read().rstrip()
        os.kill(int(pid), signal.SIGKILL)
        os.remove(pidpath)
        print("sukusho-daemon is safely killed")
        return None
    if FLAGS.test:
        mv_default(test=True)
        if not FLAGS.startdaemon:
            print("Your Sukusho is ready!!")
    if FLAGS.startdaemon:
        if os.path.isfile(pidpath):
            print("sukusho-daemon is already started.")
            return None
        print("sukusho-daemon started!")
        start_daemon(
            screenshot_daemon,
            'screenshot_daemon',
            pidpath=os.path.join(base, "python_daemon.pid"),
            logpath=os.path.join(base, "python_daemon.log"),
            kws=None
        )
    if FLAGS.exec:
        mv_default(test=False)


def sukusho():
    app.run(main)


if __name__ == "__main__":
    app.run(main)



import sys
import time
from datetime import datetime, timedelta
# pip
import daemon
import daemon.pidfile

__all__ = ['start_daemon', 'log_to_stdout', 'interval_run']


def log_to_stdout(*msg, end="\n", date=True):
    '''ログを標準(&エラー)出力

    Parameter
    ------------------------------
        *msg: object(ログ内容)
        end: str(print末尾の文字)

    Return
    ------------------------------
    '''
    if end is None:
        end = ""
    if date:
        out = str(datetime.now()) + " "
    else:
        out = ""
    for m in msg:
        out += f"{m} "
    sys.stdout.write(out + end)
    sys.stdout.flush()
    sys.stderr.flush()


def interval_run(interval):
    '''interval分ごとに関数実行
    intervalに1または60を割り切れない数を入れるとバグる(すいません)

    Parameter
    ------------------------------
        interval: int(実行間隔 -分)

    Return
    ------------------------------
        function(デコレーション後の関数)
    '''
    def _deco(f):
        def _wrapper(*args, **kws):
            while True:
                # 現在時刻と次回起動時刻
                t = datetime.now().replace(second=0, microsecond=0)
                n = t + timedelta(minutes=interval)

                # intervalの倍数ジャストに起動
                if t.minute % interval == 0:
                    f(*args, **kws)

                # 1周目 次回実行時間まで調整
                else:
                    stop = stop = interval - t.minute % interval
                    n = t + timedelta(minutes=stop)

                w = (n - datetime.now()).total_seconds()
                log_to_stdout('[INFO]', 'Sleep', w, 'seconds.')
                time.sleep(w)

        return _wrapper
    return _deco


def start_daemon(pf):
    def wrapper(func, *args, pidpath=None, logpath=None, **kws):
        '''対象の関数をデーモン化
        SIGTERMで終了 関数内でのエラーでは停止しない(1分sleep)

        Parameter
        ------------------------------
            func: function(デーモンにしたい関数)
            *args: list(funcで使う引数)
            pidpath: str(PIDファイルパス -default:fg)
            logpath: str(ログファイルパス -default:stdout)
            **kws: dict(funcで使うキーワード引数)

        Return
        ------------------------------
        '''

        # 多重起動防止
        if pidpath:
            pid = daemon.pidfile.PIDLockFile(pidpath)
            if pid.is_locked():
                raise Exception('Process is already started.')

        # PID指定がなければフォアグラウンド
        else:
            pid = None

        # ログ
        if logpath:
            std_out = open(logpath, mode='a+', encoding='utf-8')
            std_err = std_out

        # 指定がなければ標準出力
        else:
            std_out = sys.stdout
            std_err = sys.stderr

        dc = daemon.DaemonContext(
            umask=0o002,
            pidfile=pid,
            stdout=std_out,
            stderr=std_err,
        )

        # 内部で実行される
        def forever():
            pf("*Python daemon Started*")
            while True:
                try:
                    func(*args, **kws)

                # kill -SIGTERM {pid} で停止する
                except SystemExit:
                    pf('*Killed by SIGTERM.*')
                    raise

                # それ以外のエラーは無視して動き続ける
                except Exception as e:
                    pf('Uncaught exception was raised, but process continue.', e)

        with dc:
            forever()
    return wrapper
